@@ -1,13 +1,17 @@
-from carrot.connection import BrokerConnection
-from carrot.messaging import Publisher
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
+import warnings
+
+from datetime import datetime
+
+from kombu import Connection, Exchange, Producer
+from pytz import timezone
 
 from config import PulseConfiguration
 from utils import *
 
-from datetime import datetime
-from pytz import timezone
-
-import warnings
 
 # Exceptions we can raise
 class InvalidExchange(Exception):
@@ -17,8 +21,8 @@ class MalformedMessage(Exception):
     pass
 
 
-# Generic publisher class that specific publishers inherit from
 class GenericPublisher(object):
+    """Generic publisher class that specific publishers inherit from."""
 
     def __init__(self, config, exchange=None, connect=True):
         self.config = config
@@ -27,47 +31,39 @@ class GenericPublisher(object):
         if connect:
             self.connect()
 
-    # Connect to the message broker
     def connect(self):
         if not self.connection:
-            self.connection = BrokerConnection(hostname=self.config.host,
-                                               port=self.config.port,
-                                               userid=self.config.user,
-                                               password=self.config.password,
-                                               virtual_host=self.config.vhost)
+            self.connection = Connection(hostname=self.config.host,
+                                         port=self.config.port,
+                                         userid=self.config.user,
+                                         password=self.config.password,
+                                         virtual_host=self.config.vhost)
 
-    # Disconnect from the message broker
     def disconnect(self):
         if self.connection:
-            self.connection.close()
+            self.connection.release()
             self.connection = None
 
-    # Used to publish a pulse message to the proper exchange
     def publish(self, message):
+        """Publishes a pulse message to the proper exchange."""
 
-        # Make suere there is an exchange given
         if not self.exchange:
             raise InvalidExchange(self.exchange)
 
-        # Make sure there is a message given
         if not message:
             raise MalformedMessage(message)
 
-        # Have the message prepare and validate itself
         message._prepare()
 
-        # Connect to the broker if we haven't already
         if not self.connection:
             self.connect()
 
-        # Set up our broker publisher
-        self.publisher = Publisher(connection=self.connection,
-                                   exchange=self.exchange,
-                                   exchange_type="topic",
-                                   routing_key=message.routing_key)
+        producer = Producer(channel=self.connection,
+                            exchange=Exchange(self.exchange, type='topic'),
+                            routing_key=message.routing_key)
 
         # The message is actually a simple envelope format with a payload and
-        # some metadata
+        # some metadata.
         final_data = {}
         final_data['payload'] = message.data
         final_data['_meta'] = message.metadata.copy()
@@ -78,47 +74,50 @@ class GenericPublisher(object):
             'sent': time_to_string(datetime.now(timezone(self.config.broker_timezone)))
         })
 
-        # Send the message
-        self.publisher.send(final_data, serializer=self.config.serializer)
+        producer.publish(final_data, serializer=self.config.serializer)
 
-        # Close the publishing connection
-        self.publisher.close()
 
 # ------------------------------------------------------------------------------
 # Publishers for various exchanges
 # ------------------------------------------------------------------------------
 
 class PulseTestPublisher(GenericPublisher):
-    
+
     def __init__(self, **kwargs):
         super(PulseTestPublisher, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.pulse.test')
 
+
 class PulseMetaPublisher(GenericPublisher):
-    
+
     def __init__(self, **kwargs):
         super(PulseMetaPublisher, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.pulse')
 
+
 class BugzillaPublisher(GenericPublisher):
-    
+
     def __init__(self, **kwargs):
         super(BugzillaPublisher, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.bugzilla')
 
+
 class CodePublisher(GenericPublisher):
-    
+
     def __init__(self, **kwargs):
         super(CodePublisher, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.code')
 
+
 class HgPublisher(CodePublisher):
-    
+
     def __init__(self, **kwargs):
         super(HgPublisher, self).__init__(PulseConfiguration(**kwargs))
         warnings.warn('HgPublisher is now CodePublisher', DeprecationWarning)
 
+
 class BuildPublisher(GenericPublisher):
 
     def __init__(self, **kwargs):
         super(BuildPublisher, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.build')
 
+
 class QAPublisher(GenericPublisher):
 
     def __init__(self, **kwargs):
